# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..types import counter_list_params, counter_create_params, counter_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.counter_response import CounterResponse

__all__ = ["CountersResource", "AsyncCountersResource"]


class CountersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CountersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CountersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CountersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return CountersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        unit: str,
        code: str | NotGiven = NOT_GIVEN,
        product_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CounterResponse:
        """
        Create a new Counter.

        Args:
          name: Descriptive name for the Counter.

          unit: User defined label for units shown on Bill line items, and indicating to your
              customers what they are being charged for.

          code: Code for the Counter. A unique short code to identify the Counter.

          product_id: UUID of the product the Counter belongs to. _(Optional)_ - if left blank, the
              Counter is Global. A Global Counter can be used to price Plans or Plan Templates
              belonging to any Product.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/counters",
            body=maybe_transform(
                {
                    "name": name,
                    "unit": unit,
                    "code": code,
                    "product_id": product_id,
                    "version": version,
                },
                counter_create_params.CounterCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CounterResponse:
        """
        Retrieve a Counter for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/counters/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        unit: str,
        code: str | NotGiven = NOT_GIVEN,
        product_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CounterResponse:
        """
        Update Counter for the given UUID.

        Args:
          name: Descriptive name for the Counter.

          unit: User defined label for units shown on Bill line items, and indicating to your
              customers what they are being charged for.

          code: Code for the Counter. A unique short code to identify the Counter.

          product_id: UUID of the product the Counter belongs to. _(Optional)_ - if left blank, the
              Counter is Global. A Global Counter can be used to price Plans or Plan Templates
              belonging to any Product.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/counters/{id}",
            body=maybe_transform(
                {
                    "name": name,
                    "unit": unit,
                    "code": code,
                    "product_id": product_id,
                    "version": version,
                },
                counter_update_params.CounterUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        codes: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        product_id: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[CounterResponse]:
        """
        Retrieve a list of Counter entities that can be filtered by Product, Counter ID,
        or Codes.

        Args:
          codes: List of Counter codes to retrieve. These are unique short codes to identify each
              Counter.

          ids: List of Counter IDs to retrieve.

          next_token: NextToken for multi page retrievals.

          page_size: Number of Counters to retrieve per page

          product_id: List of Products UUIDs to retrieve Counters for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/counters",
            page=SyncCursor[CounterResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "product_id": product_id,
                    },
                    counter_list_params.CounterListParams,
                ),
            ),
            model=CounterResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CounterResponse:
        """
        Delete a Counter for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/counters/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterResponse,
        )


class AsyncCountersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCountersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCountersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCountersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncCountersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        unit: str,
        code: str | NotGiven = NOT_GIVEN,
        product_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CounterResponse:
        """
        Create a new Counter.

        Args:
          name: Descriptive name for the Counter.

          unit: User defined label for units shown on Bill line items, and indicating to your
              customers what they are being charged for.

          code: Code for the Counter. A unique short code to identify the Counter.

          product_id: UUID of the product the Counter belongs to. _(Optional)_ - if left blank, the
              Counter is Global. A Global Counter can be used to price Plans or Plan Templates
              belonging to any Product.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/counters",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "unit": unit,
                    "code": code,
                    "product_id": product_id,
                    "version": version,
                },
                counter_create_params.CounterCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CounterResponse:
        """
        Retrieve a Counter for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/counters/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        unit: str,
        code: str | NotGiven = NOT_GIVEN,
        product_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CounterResponse:
        """
        Update Counter for the given UUID.

        Args:
          name: Descriptive name for the Counter.

          unit: User defined label for units shown on Bill line items, and indicating to your
              customers what they are being charged for.

          code: Code for the Counter. A unique short code to identify the Counter.

          product_id: UUID of the product the Counter belongs to. _(Optional)_ - if left blank, the
              Counter is Global. A Global Counter can be used to price Plans or Plan Templates
              belonging to any Product.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/counters/{id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "unit": unit,
                    "code": code,
                    "product_id": product_id,
                    "version": version,
                },
                counter_update_params.CounterUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        codes: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        product_id: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[CounterResponse, AsyncCursor[CounterResponse]]:
        """
        Retrieve a list of Counter entities that can be filtered by Product, Counter ID,
        or Codes.

        Args:
          codes: List of Counter codes to retrieve. These are unique short codes to identify each
              Counter.

          ids: List of Counter IDs to retrieve.

          next_token: NextToken for multi page retrievals.

          page_size: Number of Counters to retrieve per page

          product_id: List of Products UUIDs to retrieve Counters for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/counters",
            page=AsyncCursor[CounterResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "product_id": product_id,
                    },
                    counter_list_params.CounterListParams,
                ),
            ),
            model=CounterResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CounterResponse:
        """
        Delete a Counter for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/counters/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterResponse,
        )


class CountersResourceWithRawResponse:
    def __init__(self, counters: CountersResource) -> None:
        self._counters = counters

        self.create = to_raw_response_wrapper(
            counters.create,
        )
        self.retrieve = to_raw_response_wrapper(
            counters.retrieve,
        )
        self.update = to_raw_response_wrapper(
            counters.update,
        )
        self.list = to_raw_response_wrapper(
            counters.list,
        )
        self.delete = to_raw_response_wrapper(
            counters.delete,
        )


class AsyncCountersResourceWithRawResponse:
    def __init__(self, counters: AsyncCountersResource) -> None:
        self._counters = counters

        self.create = async_to_raw_response_wrapper(
            counters.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            counters.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            counters.update,
        )
        self.list = async_to_raw_response_wrapper(
            counters.list,
        )
        self.delete = async_to_raw_response_wrapper(
            counters.delete,
        )


class CountersResourceWithStreamingResponse:
    def __init__(self, counters: CountersResource) -> None:
        self._counters = counters

        self.create = to_streamed_response_wrapper(
            counters.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            counters.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            counters.update,
        )
        self.list = to_streamed_response_wrapper(
            counters.list,
        )
        self.delete = to_streamed_response_wrapper(
            counters.delete,
        )


class AsyncCountersResourceWithStreamingResponse:
    def __init__(self, counters: AsyncCountersResource) -> None:
        self._counters = counters

        self.create = async_to_streamed_response_wrapper(
            counters.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            counters.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            counters.update,
        )
        self.list = async_to_streamed_response_wrapper(
            counters.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            counters.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["DataFieldResponse"]


class DataFieldResponse(BaseModel):
    category: Literal["WHO", "WHERE", "WHAT", "OTHER", "METADATA", "MEASURE", "INCOME", "COST"]
    """The type of field (WHO, WHAT, WHERE, MEASURE, METADATA, INCOME, COST, OTHER)."""

    code: str
    """Short code to identify the field

    **NOTE:** Code has a maximum length of 80 characters and can only contain
    letters, numbers, underscore, and the dollar character, and must not start with
    a number.
    """

    name: str
    """Descriptive name of the field."""

    unit: Optional[str] = None
    """The units to measure the data with.

    Should conform to _Unified Code for Units of Measure_ (UCUM). Required only for
    numeric field categories.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BalanceCreateParams"]


class BalanceCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_id: Required[Annotated[str, PropertyInfo(alias="accountId")]]
    """The unique identifier (UUID) for the end customer Account."""

    currency: Required[str]
    """The currency code used for the Balance amount. For example: USD, GBP or EUR."""

    end_date: Required[Annotated[Union[str, datetime], PropertyInfo(alias="endDate", format="iso8601")]]
    """
    The date _(in ISO 8601 format)_ after which the Balance will no longer be active
    for the Account.

    **Note:** You can use the `rolloverEndDate` request parameter to define an
    extended grace period for continued draw-down against the Balance if any amount
    remains when the specified `endDate` is reached.
    """

    start_date: Required[Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]]
    """The date _(in ISO 8601 format)_ when the Balance becomes active."""

    balance_draw_down_description: Annotated[str, PropertyInfo(alias="balanceDrawDownDescription")]
    """A description for the bill line items for draw-down charges against the Balance.

    _(Optional)._
    """

    code: str
    """Unique short code for the Balance."""

    consumptions_accounting_product_id: Annotated[str, PropertyInfo(alias="consumptionsAccountingProductId")]
    """
    Optional Product ID this Balance Consumptions should be attributed to for
    accounting purposes
    """

    description: str
    """A description of the Balance."""

    fees_accounting_product_id: Annotated[str, PropertyInfo(alias="feesAccountingProductId")]
    """
    Optional Product ID this Balance Fees should be attributed to for accounting
    purposes
    """

    line_item_types: Annotated[
        List[
            Literal[
                "STANDING_CHARGE", "USAGE", "MINIMUM_SPEND", "COUNTER_RUNNING_TOTAL_CHARGE", "COUNTER_ADJUSTMENT_DEBIT"
            ]
        ],
        PropertyInfo(alias="lineItemTypes"),
    ]
    """
    Specify the line item charge types that can draw-down at billing against the
    Balance amount. Options are:

    - `"MINIMUM_SPEND"`
    - `"STANDING_CHARGE"`
    - `"USAGE"`
    - `"COUNTER_RUNNING_TOTAL_CHARGE"`
    - `"COUNTER_ADJUSTMENT_DEBIT"`

    **NOTE:** If no charge types are specified, by default _all types_ can draw-down
    against the Balance amount at billing.
    """

    name: str
    """The official name for the Balance."""

    overage_description: Annotated[str, PropertyInfo(alias="overageDescription")]
    """A description for Bill line items overage charges."""

    overage_surcharge_percent: Annotated[float, PropertyInfo(alias="overageSurchargePercent")]
    """
    Define a surcharge level, as a percentage of regular usage rating, applied to
    overages _(usage charges that exceed the Balance amount)_. For example, if the
    regular usage rate is $10 per unit of usage consumed and
    `overageSurchargePercent` is set at 10%, then any usage charged above the
    original Balance amount is charged at $11 per unit of usage.
    """

    product_ids: Annotated[List[str], PropertyInfo(alias="productIds")]
    """
    Specify the Products whose consumption charges due at billing can be drawn-down
    against the Balance amount.

    **Note:** If you don't specify any Products for Balance draw-down, by default
    the consumption charges for any Product the Account consumes will be drawn-down
    against the Balance amount.
    """

    rollover_amount: Annotated[float, PropertyInfo(alias="rolloverAmount")]
    """
    The maximum amount that can be carried over past the Balance end date for
    draw-down at billing if there is any unused Balance amount when the end date is
    reached. Works with `rolloverEndDate` to define the amount and duration of a
    Balance "grace period". _(Optional)_

    **Notes:**

    - If you leave `rolloverAmount` empty and only enter a `rolloverEndDate`, any
      amount left over after the Balance end date is reached will be drawn-down
      against up to the specified `rolloverEndDate`.
    - You must enter a `rolloverEndDate`. If you only enter a `rolloverAmount`
      without entering a `rolloverEndDate`, you'll receive an error when trying to
      create or update the Balance.
    - If you don't want to grant any grace period for outstanding Balance amounts,
      then do not use `rolloverAmount` and `rolloverEndDate`.
    """

    rollover_end_date: Annotated[Union[str, datetime], PropertyInfo(alias="rolloverEndDate", format="iso8601")]
    """
    The end date _(in ISO 8601 format)_ for the grace period during which unused
    Balance amounts can be carried over and drawn-down against at billing.

    **Note:** Use `rolloverAmount` if you want to specify a maximum amount that can
    be carried over and made available for draw-down.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ...types.bills import line_item_list_params
from ..._base_client import AsyncPaginator, make_request_options
from ...types.bills.line_item_response import LineItemResponse

__all__ = ["LineItemsResource", "AsyncLineItemsResource"]


class LineItemsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LineItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return LineItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LineItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return LineItemsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        bill_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LineItemResponse:
        """
        Retrieves a specific line item within a Bill.

        This endpoint retrieves the line item given by its unique identifier (UUID) from
        a specific Bill.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not bill_id:
            raise ValueError(f"Expected a non-empty value for `bill_id` but received {bill_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/bills/{bill_id}/lineitems/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LineItemResponse,
        )

    def list(
        self,
        bill_id: str,
        *,
        org_id: str | None = None,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[LineItemResponse]:
        """
        Lists all the line items for a specific Bill.

        This endpoint retrieves a list of line items for the given Bill within the
        specified Organization. The list can also be paginated for easier management.
        The line items returned in the list include individual charges, discounts, or
        adjustments within a Bill.

        Args:
          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              line items in a paginated list.

          page_size: Specifies the maximum number of line items to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not bill_id:
            raise ValueError(f"Expected a non-empty value for `bill_id` but received {bill_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/bills/{bill_id}/lineitems",
            page=SyncCursor[LineItemResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    line_item_list_params.LineItemListParams,
                ),
            ),
            model=LineItemResponse,
        )


class AsyncLineItemsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLineItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLineItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLineItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncLineItemsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        bill_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LineItemResponse:
        """
        Retrieves a specific line item within a Bill.

        This endpoint retrieves the line item given by its unique identifier (UUID) from
        a specific Bill.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not bill_id:
            raise ValueError(f"Expected a non-empty value for `bill_id` but received {bill_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/bills/{bill_id}/lineitems/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LineItemResponse,
        )

    def list(
        self,
        bill_id: str,
        *,
        org_id: str | None = None,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[LineItemResponse, AsyncCursor[LineItemResponse]]:
        """
        Lists all the line items for a specific Bill.

        This endpoint retrieves a list of line items for the given Bill within the
        specified Organization. The list can also be paginated for easier management.
        The line items returned in the list include individual charges, discounts, or
        adjustments within a Bill.

        Args:
          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              line items in a paginated list.

          page_size: Specifies the maximum number of line items to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not bill_id:
            raise ValueError(f"Expected a non-empty value for `bill_id` but received {bill_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/bills/{bill_id}/lineitems",
            page=AsyncCursor[LineItemResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    line_item_list_params.LineItemListParams,
                ),
            ),
            model=LineItemResponse,
        )


class LineItemsResourceWithRawResponse:
    def __init__(self, line_items: LineItemsResource) -> None:
        self._line_items = line_items

        self.retrieve = to_raw_response_wrapper(
            line_items.retrieve,
        )
        self.list = to_raw_response_wrapper(
            line_items.list,
        )


class AsyncLineItemsResourceWithRawResponse:
    def __init__(self, line_items: AsyncLineItemsResource) -> None:
        self._line_items = line_items

        self.retrieve = async_to_raw_response_wrapper(
            line_items.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            line_items.list,
        )


class LineItemsResourceWithStreamingResponse:
    def __init__(self, line_items: LineItemsResource) -> None:
        self._line_items = line_items

        self.retrieve = to_streamed_response_wrapper(
            line_items.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            line_items.list,
        )


class AsyncLineItemsResourceWithStreamingResponse:
    def __init__(self, line_items: AsyncLineItemsResource) -> None:
        self._line_items = line_items

        self.retrieve = async_to_streamed_response_wrapper(
            line_items.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            line_items.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .account_response import AccountResponse

__all__ = ["AccountSearchResponse"]


class AccountSearchResponse(BaseModel):
    data: Optional[List[AccountResponse]] = None

    next_token: Optional[str] = FieldInfo(alias="nextToken", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union

import httpx

from ..types import product_list_params, product_create_params, product_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.product_response import ProductResponse

__all__ = ["ProductsResource", "AsyncProductsResource"]


class ProductsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProductsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ProductsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProductsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return ProductsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProductResponse:
        """
        Create a new Product.

        This endpoint creates a new Product within the specified Organization. The
        details of the Product are provided in the request body.

        Args:
          code: A unique short code to identify the Product. It should not contain control
              chracters or spaces.

          name: Descriptive name for the Product providing context and information.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/products",
            body=maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                product_create_params.ProductCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProductResponse:
        """
        Retrieve a Product with the given UUID.

        This endpoint retrieves the details of a specific Product within a specified
        Organization, using the Product UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProductResponse:
        """
        Update a Product with the given UUID.

        This endpoint updates the details of a specific Product within a specified
        Organization, using the Product UUID. The updated details are provided in the
        request body.

        **Note:** If you have created Custom Fields for a Product, when you use this
        endpoint to update the Product use the `customFields` parameter to preserve
        those Custom Fields. If you omit them from the update request, they will be
        lost.

        Args:
          code: A unique short code to identify the Product. It should not contain control
              chracters or spaces.

          name: Descriptive name for the Product providing context and information.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/products/{id}",
            body=maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                product_update_params.ProductUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[ProductResponse]:
        """
        Retrieve a list of Products.

        This endpoint retrieves a list of all the Products within a specified
        Organization. The list can be paginated, and supports filtering by specific
        Product IDs.

        Args:
          ids: List of specific Product UUIDs to retrieve.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Products in a paginated list.

          page_size: Specifies the maximum number of Products to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/products",
            page=SyncCursor[ProductResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    product_list_params.ProductListParams,
                ),
            ),
            model=ProductResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProductResponse:
        """
        Delete a Product with the given UUID.

        This endpoint deletes a specific Product within a specified Organization, using
        the Product UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductResponse,
        )


class AsyncProductsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProductsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProductsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProductsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncProductsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProductResponse:
        """
        Create a new Product.

        This endpoint creates a new Product within the specified Organization. The
        details of the Product are provided in the request body.

        Args:
          code: A unique short code to identify the Product. It should not contain control
              chracters or spaces.

          name: Descriptive name for the Product providing context and information.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/products",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                product_create_params.ProductCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProductResponse:
        """
        Retrieve a Product with the given UUID.

        This endpoint retrieves the details of a specific Product within a specified
        Organization, using the Product UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProductResponse:
        """
        Update a Product with the given UUID.

        This endpoint updates the details of a specific Product within a specified
        Organization, using the Product UUID. The updated details are provided in the
        request body.

        **Note:** If you have created Custom Fields for a Product, when you use this
        endpoint to update the Product use the `customFields` parameter to preserve
        those Custom Fields. If you omit them from the update request, they will be
        lost.

        Args:
          code: A unique short code to identify the Product. It should not contain control
              chracters or spaces.

          name: Descriptive name for the Product providing context and information.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/products/{id}",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                product_update_params.ProductUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ProductResponse, AsyncCursor[ProductResponse]]:
        """
        Retrieve a list of Products.

        This endpoint retrieves a list of all the Products within a specified
        Organization. The list can be paginated, and supports filtering by specific
        Product IDs.

        Args:
          ids: List of specific Product UUIDs to retrieve.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Products in a paginated list.

          page_size: Specifies the maximum number of Products to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/products",
            page=AsyncCursor[ProductResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    product_list_params.ProductListParams,
                ),
            ),
            model=ProductResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProductResponse:
        """
        Delete a Product with the given UUID.

        This endpoint deletes a specific Product within a specified Organization, using
        the Product UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/products/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductResponse,
        )


class ProductsResourceWithRawResponse:
    def __init__(self, products: ProductsResource) -> None:
        self._products = products

        self.create = to_raw_response_wrapper(
            products.create,
        )
        self.retrieve = to_raw_response_wrapper(
            products.retrieve,
        )
        self.update = to_raw_response_wrapper(
            products.update,
        )
        self.list = to_raw_response_wrapper(
            products.list,
        )
        self.delete = to_raw_response_wrapper(
            products.delete,
        )


class AsyncProductsResourceWithRawResponse:
    def __init__(self, products: AsyncProductsResource) -> None:
        self._products = products

        self.create = async_to_raw_response_wrapper(
            products.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            products.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            products.update,
        )
        self.list = async_to_raw_response_wrapper(
            products.list,
        )
        self.delete = async_to_raw_response_wrapper(
            products.delete,
        )


class ProductsResourceWithStreamingResponse:
    def __init__(self, products: ProductsResource) -> None:
        self._products = products

        self.create = to_streamed_response_wrapper(
            products.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            products.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            products.update,
        )
        self.list = to_streamed_response_wrapper(
            products.list,
        )
        self.delete = to_streamed_response_wrapper(
            products.delete,
        )


class AsyncProductsResourceWithStreamingResponse:
    def __init__(self, products: AsyncProductsResource) -> None:
        self._products = products

        self.create = async_to_streamed_response_wrapper(
            products.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            products.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            products.update,
        )
        self.list = async_to_streamed_response_wrapper(
            products.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            products.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .data_field_response import DataFieldResponse

__all__ = ["MeterResponse", "DerivedField"]


class DerivedField(DataFieldResponse):
    calculation: str
    """
    The calculation used to transform the value of submitted `dataFields` in usage
    data. Calculation can reference `dataFields`, `customFields`, or system
    `Timestamp` fields. _(Example: datafieldms datafieldgb)_
    """


class MeterResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    code: Optional[str] = None
    """Code of the Meter - unique short code used to identify the Meter."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created this meter."""

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    data_fields: Optional[List[DataFieldResponse]] = FieldInfo(alias="dataFields", default=None)
    """
    Used to submit categorized raw usage data values for ingest into the platform -
    either numeric quantitative values or non-numeric data values. At least one
    required per Meter; maximum 15 per Meter.
    """

    derived_fields: Optional[List[DerivedField]] = FieldInfo(alias="derivedFields", default=None)
    """
    Used to submit usage data values for ingest into the platform that are the
    result of a calculation performed on `dataFields`, `customFields`, or system
    `Timestamp` fields. Raw usage data is not submitted using `derivedFields`.
    Maximum 15 per Meter. _(Optional)_.
    """

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when the meter was created _(in ISO-8601 format)_."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when the meter was last modified _(in ISO-8601 format)_."""

    group_id: Optional[str] = FieldInfo(alias="groupId", default=None)
    """UUID of the MeterGroup the Meter belongs to. _(Optional)_."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified this meter."""

    name: Optional[str] = None
    """Descriptive name for the Meter."""

    product_id: Optional[str] = FieldInfo(alias="productId", default=None)
    """UUID of the Product the Meter belongs to.

    _(Optional)_ - if blank, the Meter is global.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursor, AsyncCursor
from ...._base_client import AsyncPaginator, make_request_options
from ....types.usage.file_uploads import job_list_params
from ....types.usage.file_uploads.file_upload_job_response import FileUploadJobResponse
from ....types.usage.file_uploads.job_get_original_download_url_response import JobGetOriginalDownloadURLResponse

__all__ = ["JobsResource", "AsyncJobsResource"]


class JobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> JobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return JobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> JobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return JobsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FileUploadJobResponse:
        """
        Get the file upload job response using the UUID of the file upload job.

        Part of the file upload service for measurements ingest.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/fileuploads/measurements/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileUploadJobResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        date_created_end: str | NotGiven = NOT_GIVEN,
        date_created_start: str | NotGiven = NOT_GIVEN,
        file_key: Optional[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[FileUploadJobResponse]:
        """Lists the File Upload jobs.

        Part of the File Upload service for measurements
        ingest:

        - You can use the `dateCreatedStart` and `dateCreatedEnd` optional Query
          parameters to define a date range to filter the File Uploads jobs returned for
          this call.
        - If `dateCreatedStart` and `dateCreatedEnd` Query parameters are not used, then
          all File Upload jobs are returned.

        Args:
          date_created_end: Include only File Upload jobs created before this date. Required format is
              ISO-8601: yyyy-MM-dd'T'HH:mm:ss'Z'

          date_created_start: Include only File Upload jobs created on or after this date. Required format is
              ISO-8601: yyyy-MM-dd'T'HH:mm:ss'Z'

          file_key: <<deprecated>>

          next_token: `nextToken` for multi page retrievals.

          page_size: Number of File Upload jobs to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/fileuploads/measurements/jobs",
            page=SyncCursor[FileUploadJobResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_created_end": date_created_end,
                        "date_created_start": date_created_start,
                        "file_key": file_key,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            model=FileUploadJobResponse,
        )

    def get_original_download_url(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> JobGetOriginalDownloadURLResponse:
        """
        Use the original file upload job id to obtain a download URL, which you can then
        use to retrieve the file you originally uploaded to the file upload service:

        - A download URL is returned together with a download job id.
        - You can then use a `GET` using the returned download URL as the endpoint to
          retrieve the file you originally uploaded.

        Part of the file upload service for submitting measurements data files.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/fileuploads/measurements/jobs/{id}/original",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobGetOriginalDownloadURLResponse,
        )


class AsyncJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncJobsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FileUploadJobResponse:
        """
        Get the file upload job response using the UUID of the file upload job.

        Part of the file upload service for measurements ingest.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/fileuploads/measurements/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileUploadJobResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        date_created_end: str | NotGiven = NOT_GIVEN,
        date_created_start: str | NotGiven = NOT_GIVEN,
        file_key: Optional[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[FileUploadJobResponse, AsyncCursor[FileUploadJobResponse]]:
        """Lists the File Upload jobs.

        Part of the File Upload service for measurements
        ingest:

        - You can use the `dateCreatedStart` and `dateCreatedEnd` optional Query
          parameters to define a date range to filter the File Uploads jobs returned for
          this call.
        - If `dateCreatedStart` and `dateCreatedEnd` Query parameters are not used, then
          all File Upload jobs are returned.

        Args:
          date_created_end: Include only File Upload jobs created before this date. Required format is
              ISO-8601: yyyy-MM-dd'T'HH:mm:ss'Z'

          date_created_start: Include only File Upload jobs created on or after this date. Required format is
              ISO-8601: yyyy-MM-dd'T'HH:mm:ss'Z'

          file_key: <<deprecated>>

          next_token: `nextToken` for multi page retrievals.

          page_size: Number of File Upload jobs to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/fileuploads/measurements/jobs",
            page=AsyncCursor[FileUploadJobResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_created_end": date_created_end,
                        "date_created_start": date_created_start,
                        "file_key": file_key,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            model=FileUploadJobResponse,
        )

    async def get_original_download_url(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> JobGetOriginalDownloadURLResponse:
        """
        Use the original file upload job id to obtain a download URL, which you can then
        use to retrieve the file you originally uploaded to the file upload service:

        - A download URL is returned together with a download job id.
        - You can then use a `GET` using the returned download URL as the endpoint to
          retrieve the file you originally uploaded.

        Part of the file upload service for submitting measurements data files.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/fileuploads/measurements/jobs/{id}/original",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobGetOriginalDownloadURLResponse,
        )


class JobsResourceWithRawResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.retrieve = to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            jobs.list,
        )
        self.get_original_download_url = to_raw_response_wrapper(
            jobs.get_original_download_url,
        )


class AsyncJobsResourceWithRawResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.retrieve = async_to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            jobs.list,
        )
        self.get_original_download_url = async_to_raw_response_wrapper(
            jobs.get_original_download_url,
        )


class JobsResourceWithStreamingResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.retrieve = to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            jobs.list,
        )
        self.get_original_download_url = to_streamed_response_wrapper(
            jobs.get_original_download_url,
        )


class AsyncJobsResourceWithStreamingResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.retrieve = async_to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            jobs.list,
        )
        self.get_original_download_url = async_to_streamed_response_wrapper(
            jobs.get_original_download_url,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .shared import SetString as SetString, PricingBand as PricingBand, CurrencyConversion as CurrencyConversion
from .address import Address as Address
from .balance import Balance as Balance
from .webhook import Webhook as Webhook
from .address_param import AddressParam as AddressParam
from .bill_response import BillResponse as BillResponse
from .plan_response import PlanResponse as PlanResponse
from .user_response import UserResponse as UserResponse
from .commitment_fee import CommitmentFee as CommitmentFee
from .event_response import EventResponse as EventResponse
from .meter_response import MeterResponse as MeterResponse
from .ad_hoc_response import AdHocResponse as AdHocResponse
from .account_response import AccountResponse as AccountResponse
from .bill_list_params import BillListParams as BillListParams
from .counter_response import CounterResponse as CounterResponse
from .plan_list_params import PlanListParams as PlanListParams
from .pricing_response import PricingResponse as PricingResponse
from .product_response import ProductResponse as ProductResponse
from .user_list_params import UserListParams as UserListParams
from .user_me_response import UserMeResponse as UserMeResponse
from .bill_job_response import BillJobResponse as BillJobResponse
from .contract_response import ContractResponse as ContractResponse
from .currency_response import CurrencyResponse as CurrencyResponse
from .event_list_params import EventListParams as EventListParams
from .meter_list_params import MeterListParams as MeterListParams
from .bill_search_params import BillSearchParams as BillSearchParams
from .plan_create_params import PlanCreateParams as PlanCreateParams
from .plan_update_params import PlanUpdateParams as PlanUpdateParams
from .usage_query_params import UsageQueryParams as UsageQueryParams
from .user_update_params import UserUpdateParams as UserUpdateParams
from .account_list_params import AccountListParams as AccountListParams
from .balance_list_params import BalanceListParams as BalanceListParams
from .bill_approve_params import BillApproveParams as BillApproveParams
from .commitment_response import CommitmentResponse as CommitmentResponse
from .counter_list_params import CounterListParams as CounterListParams
from .data_field_response import DataFieldResponse as DataFieldResponse
from .meter_create_params import MeterCreateParams as MeterCreateParams
from .meter_update_params import MeterUpdateParams as MeterUpdateParams
from .plan_group_response import PlanGroupResponse as PlanGroupResponse
from .pricing_list_params import PricingListParams as PricingListParams
from .product_list_params import ProductListParams as ProductListParams
from .usage_submit_params import UsageSubmitParams as UsageSubmitParams
from .webhook_list_params import WebhookListParams as WebhookListParams
from .aggregation_response import AggregationResponse as AggregationResponse
from .bill_config_response import BillConfigResponse as BillConfigResponse
from .bill_job_list_params import BillJobListParams as BillJobListParams
from .bill_search_response import BillSearchResponse as BillSearchResponse
from .commitment_fee_param import CommitmentFeeParam as CommitmentFeeParam
from .contract_list_params import ContractListParams as ContractListParams
from .currency_list_params import CurrencyListParams as CurrencyListParams
from .usage_query_response import UsageQueryResponse as UsageQueryResponse
from .account_create_params import AccountCreateParams as AccountCreateParams
from .account_plan_response import AccountPlanResponse as AccountPlanResponse
from .account_search_params import AccountSearchParams as AccountSearchParams
from .account_update_params import AccountUpdateParams as AccountUpdateParams
from .balance_create_params import BalanceCreateParams as BalanceCreateParams
from .balance_update_params import BalanceUpdateParams as BalanceUpdateParams
from .bill_approve_response import BillApproveResponse as BillApproveResponse
from .counter_create_params import CounterCreateParams as CounterCreateParams
from .counter_update_params import CounterUpdateParams as CounterUpdateParams
from .debit_reason_response import DebitReasonResponse as DebitReasonResponse
from .download_url_response import DownloadURLResponse as DownloadURLResponse
from .pricing_create_params import PricingCreateParams as PricingCreateParams
from .pricing_update_params import PricingUpdateParams as PricingUpdateParams
from .product_create_params import ProductCreateParams as ProductCreateParams
from .product_update_params import ProductUpdateParams as ProductUpdateParams
from .webhook_create_params import WebhookCreateParams as WebhookCreateParams
from .webhook_update_params import WebhookUpdateParams as WebhookUpdateParams
from .bill_job_create_params import BillJobCreateParams as BillJobCreateParams
from .commitment_list_params import CommitmentListParams as CommitmentListParams
from .contract_create_params import ContractCreateParams as ContractCreateParams
from .contract_update_params import ContractUpdateParams as ContractUpdateParams
from .credit_reason_response import CreditReasonResponse as CreditReasonResponse
from .currency_create_params import CurrencyCreateParams as CurrencyCreateParams
from .currency_update_params import CurrencyUpdateParams as CurrencyUpdateParams
from .custom_fields_response import CustomFieldsResponse as CustomFieldsResponse
from .plan_group_list_params import PlanGroupListParams as PlanGroupListParams
from .plan_template_response import PlanTemplateResponse as PlanTemplateResponse
from .account_search_response import AccountSearchResponse as AccountSearchResponse
from .aggregation_list_params import AggregationListParams as AggregationListParams
from .event_get_fields_params import EventGetFieldsParams as EventGetFieldsParams
from .resource_group_response import ResourceGroupResponse as ResourceGroupResponse
from .webhook_create_response import WebhookCreateResponse as WebhookCreateResponse
from .webhook_update_response import WebhookUpdateResponse as WebhookUpdateResponse
from .account_plan_list_params import AccountPlanListParams as AccountPlanListParams
from .commitment_create_params import CommitmentCreateParams as CommitmentCreateParams
from .commitment_search_params import CommitmentSearchParams as CommitmentSearchParams
from .commitment_update_params import CommitmentUpdateParams as CommitmentUpdateParams
from .counter_pricing_response import CounterPricingResponse as CounterPricingResponse
from .debit_reason_list_params import DebitReasonListParams as DebitReasonListParams
from .event_get_types_response import EventGetTypesResponse as EventGetTypesResponse
from .plan_group_create_params import PlanGroupCreateParams as PlanGroupCreateParams
from .plan_group_link_response import PlanGroupLinkResponse as PlanGroupLinkResponse
from .plan_group_update_params import PlanGroupUpdateParams as PlanGroupUpdateParams
from .aggregation_create_params import AggregationCreateParams as AggregationCreateParams
from .aggregation_update_params import AggregationUpdateParams as AggregationUpdateParams
from .bill_config_update_params import BillConfigUpdateParams as BillConfigUpdateParams
from .bill_update_status_params import BillUpdateStatusParams as BillUpdateStatusParams
from .credit_reason_list_params import CreditReasonListParams as CreditReasonListParams
from .data_field_response_param import DataFieldResponseParam as DataFieldResponseParam
from .event_get_fields_response import EventGetFieldsResponse as EventGetFieldsResponse
from .external_mapping_response import ExternalMappingResponse as ExternalMappingResponse
from .plan_template_list_params import PlanTemplateListParams as PlanTemplateListParams
from .transaction_type_response import TransactionTypeResponse as TransactionTypeResponse
from .webhook_set_active_params import WebhookSetActiveParams as WebhookSetActiveParams
from .account_plan_create_params import AccountPlanCreateParams as AccountPlanCreateParams
from .account_plan_update_params import AccountPlanUpdateParams as AccountPlanUpdateParams
from .commitment_search_response import CommitmentSearchResponse as CommitmentSearchResponse
from .custom_field_update_params import CustomFieldUpdateParams as CustomFieldUpdateParams
from .debit_reason_create_params import DebitReasonCreateParams as DebitReasonCreateParams
from .debit_reason_update_params import DebitReasonUpdateParams as DebitReasonUpdateParams
from .permission_policy_response import PermissionPolicyResponse as PermissionPolicyResponse
from .resource_group_list_params import ResourceGroupListParams as ResourceGroupListParams
from .account_get_children_params import AccountGetChildrenParams as AccountGetChildrenParams
from .bill_job_recalculate_params import BillJobRecalculateParams as BillJobRecalculateParams
from .counter_adjustment_response import CounterAdjustmentResponse as CounterAdjustmentResponse
from .counter_pricing_list_params import CounterPricingListParams as CounterPricingListParams
from .credit_reason_create_params import CreditReasonCreateParams as CreditReasonCreateParams
from .credit_reason_update_params import CreditReasonUpdateParams as CreditReasonUpdateParams
from .plan_group_link_list_params import PlanGroupLinkListParams as PlanGroupLinkListParams
from .plan_template_create_params import PlanTemplateCreateParams as PlanTemplateCreateParams
from .plan_template_update_params import PlanTemplateUpdateParams as PlanTemplateUpdateParams
from .user_get_permissions_params import UserGetPermissionsParams as UserGetPermissionsParams
from .user_get_user_groups_params import UserGetUserGroupsParams as UserGetUserGroupsParams
from .webhook_set_active_response import WebhookSetActiveResponse as WebhookSetActiveResponse
from .external_mapping_list_params import ExternalMappingListParams as ExternalMappingListParams
from .organization_config_response import OrganizationConfigResponse as OrganizationConfigResponse
from .resource_group_create_params import ResourceGroupCreateParams as ResourceGroupCreateParams
from .resource_group_update_params import ResourceGroupUpdateParams as ResourceGroupUpdateParams
from .submit_measurements_response import SubmitMeasurementsResponse as SubmitMeasurementsResponse
from .transaction_type_list_params import TransactionTypeListParams as TransactionTypeListParams
from .compound_aggregation_response import CompoundAggregationResponse as CompoundAggregationResponse
from .counter_pricing_create_params import CounterPricingCreateParams as CounterPricingCreateParams
from .counter_pricing_update_params import CounterPricingUpdateParams as CounterPricingUpdateParams
from .permission_policy_list_params import PermissionPolicyListParams as PermissionPolicyListParams
from .permission_statement_response import PermissionStatementResponse as PermissionStatementResponse
from .plan_group_link_create_params import PlanGroupLinkCreateParams as PlanGroupLinkCreateParams
from .plan_group_link_update_params import PlanGroupLinkUpdateParams as PlanGroupLinkUpdateParams
from .counter_adjustment_list_params import CounterAdjustmentListParams as CounterAdjustmentListParams
from .external_mapping_create_params import ExternalMappingCreateParams as ExternalMappingCreateParams
from .external_mapping_update_params import ExternalMappingUpdateParams as ExternalMappingUpdateParams
from .transaction_type_create_params import TransactionTypeCreateParams as TransactionTypeCreateParams
from .transaction_type_update_params import TransactionTypeUpdateParams as TransactionTypeUpdateParams
from .ad_hoc_usage_data_request_param import AdHocUsageDataRequestParam as AdHocUsageDataRequestParam
from .data_export_create_adhoc_params import DataExportCreateAdhocParams as DataExportCreateAdhocParams
from .permission_policy_create_params import PermissionPolicyCreateParams as PermissionPolicyCreateParams
from .permission_policy_update_params import PermissionPolicyUpdateParams as PermissionPolicyUpdateParams
from .compound_aggregation_list_params import CompoundAggregationListParams as CompoundAggregationListParams
from .counter_adjustment_create_params import CounterAdjustmentCreateParams as CounterAdjustmentCreateParams
from .counter_adjustment_update_params import CounterAdjustmentUpdateParams as CounterAdjustmentUpdateParams
from .m3ter_signed_credentials_request import M3terSignedCredentialsRequest as M3terSignedCredentialsRequest
from .m3ter_signed_credentials_response import M3terSignedCredentialsResponse as M3terSignedCredentialsResponse
from .organization_config_update_params import OrganizationConfigUpdateParams as OrganizationConfigUpdateParams
from .compound_aggregation_create_params import CompoundAggregationCreateParams as CompoundAggregationCreateParams
from .compound_aggregation_update_params import CompoundAggregationUpdateParams as CompoundAggregationUpdateParams
from .integration_configuration_response import IntegrationConfigurationResponse as IntegrationConfigurationResponse
from .resource_group_add_resource_params import ResourceGroupAddResourceParams as ResourceGroupAddResourceParams
from .notification_configuration_response import NotificationConfigurationResponse as NotificationConfigurationResponse
from .permission_statement_response_param import PermissionStatementResponseParam as PermissionStatementResponseParam
from .resource_group_list_contents_params import ResourceGroupListContentsParams as ResourceGroupListContentsParams
from .permission_policy_add_to_user_params import PermissionPolicyAddToUserParams as PermissionPolicyAddToUserParams
from .ad_hoc_operational_data_request_param import AdHocOperationalDataRequestParam as AdHocOperationalDataRequestParam
from .integration_configuration_list_params import (
    IntegrationConfigurationListParams as IntegrationConfigurationListParams,
)
from .resource_group_list_contents_response import (
    ResourceGroupListContentsResponse as ResourceGroupListContentsResponse,
)
from .resource_group_remove_resource_params import (
    ResourceGroupRemoveResourceParams as ResourceGroupRemoveResourceParams,
)
from .authentication_get_bearer_token_params import (
    AuthenticationGetBearerTokenParams as AuthenticationGetBearerTokenParams,
)
from .m3ter_signed_credentials_request_param import (
    M3terSignedCredentialsRequestParam as M3terSignedCredentialsRequestParam,
)
from .notification_configuration_list_params import (
    NotificationConfigurationListParams as NotificationConfigurationListParams,
)
from .permission_policy_add_to_user_response import (
    PermissionPolicyAddToUserResponse as PermissionPolicyAddToUserResponse,
)
from .resource_group_list_permissions_params import (
    ResourceGroupListPermissionsParams as ResourceGroupListPermissionsParams,
)
from .scheduled_event_configuration_response import (
    ScheduledEventConfigurationResponse as ScheduledEventConfigurationResponse,
)
from .integration_configuration_create_params import (
    IntegrationConfigurationCreateParams as IntegrationConfigurationCreateParams,
)
from .integration_configuration_list_response import (
    IntegrationConfigurationListResponse as IntegrationConfigurationListResponse,
)
from .integration_configuration_update_params import (
    IntegrationConfigurationUpdateParams as IntegrationConfigurationUpdateParams,
)
from .account_end_date_billing_entities_params import (
    AccountEndDateBillingEntitiesParams as AccountEndDateBillingEntitiesParams,
)
from .authentication_get_bearer_token_response import (
    AuthenticationGetBearerTokenResponse as AuthenticationGetBearerTokenResponse,
)
from .notification_configuration_create_params import (
    NotificationConfigurationCreateParams as NotificationConfigurationCreateParams,
)
from .notification_configuration_update_params import (
    NotificationConfigurationUpdateParams as NotificationConfigurationUpdateParams,
)
from .contract_end_date_billing_entities_params import (
    ContractEndDateBillingEntitiesParams as ContractEndDateBillingEntitiesParams,
)
from .integration_configuration_create_response import (
    IntegrationConfigurationCreateResponse as IntegrationConfigurationCreateResponse,
)
from .integration_configuration_delete_response import (
    IntegrationConfigurationDeleteResponse as IntegrationConfigurationDeleteResponse,
)
from .integration_configuration_enable_response import (
    IntegrationConfigurationEnableResponse as IntegrationConfigurationEnableResponse,
)
from .integration_configuration_update_response import (
    IntegrationConfigurationUpdateResponse as IntegrationConfigurationUpdateResponse,
)
from .permission_policy_remove_from_user_params import (
    PermissionPolicyRemoveFromUserParams as PermissionPolicyRemoveFromUserParams,
)
from .scheduled_event_configuration_list_params import (
    ScheduledEventConfigurationListParams as ScheduledEventConfigurationListParams,
)
from .account_end_date_billing_entities_response import (
    AccountEndDateBillingEntitiesResponse as AccountEndDateBillingEntitiesResponse,
)
from .permission_policy_add_to_user_group_params import (
    PermissionPolicyAddToUserGroupParams as PermissionPolicyAddToUserGroupParams,
)
from .contract_end_date_billing_entities_response import (
    ContractEndDateBillingEntitiesResponse as ContractEndDateBillingEntitiesResponse,
)
from .permission_policy_remove_from_user_response import (
    PermissionPolicyRemoveFromUserResponse as PermissionPolicyRemoveFromUserResponse,
)
from .scheduled_event_configuration_create_params import (
    ScheduledEventConfigurationCreateParams as ScheduledEventConfigurationCreateParams,
)
from .scheduled_event_configuration_update_params import (
    ScheduledEventConfigurationUpdateParams as ScheduledEventConfigurationUpdateParams,
)
from .usage_get_failed_ingest_download_url_params import (
    UsageGetFailedIngestDownloadURLParams as UsageGetFailedIngestDownloadURLParams,
)
from .external_mapping_list_by_m3ter_entity_params import (
    ExternalMappingListByM3terEntityParams as ExternalMappingListByM3terEntityParams,
)
from .permission_policy_add_to_service_user_params import (
    PermissionPolicyAddToServiceUserParams as PermissionPolicyAddToServiceUserParams,
)
from .permission_policy_add_to_support_user_params import (
    PermissionPolicyAddToSupportUserParams as PermissionPolicyAddToSupportUserParams,
)
from .permission_policy_add_to_user_group_response import (
    PermissionPolicyAddToUserGroupResponse as PermissionPolicyAddToUserGroupResponse,
)
from .integration_configuration_get_by_entity_params import (
    IntegrationConfigurationGetByEntityParams as IntegrationConfigurationGetByEntityParams,
)
from .permission_policy_add_to_service_user_response import (
    PermissionPolicyAddToServiceUserResponse as PermissionPolicyAddToServiceUserResponse,
)
from .permission_policy_add_to_support_user_response import (
    PermissionPolicyAddToSupportUserResponse as PermissionPolicyAddToSupportUserResponse,
)
from .external_mapping_list_by_external_entity_params import (
    ExternalMappingListByExternalEntityParams as ExternalMappingListByExternalEntityParams,
)
from .permission_policy_remove_from_user_group_params import (
    PermissionPolicyRemoveFromUserGroupParams as PermissionPolicyRemoveFromUserGroupParams,
)
from .permission_policy_remove_from_service_user_params import (
    PermissionPolicyRemoveFromServiceUserParams as PermissionPolicyRemoveFromServiceUserParams,
)
from .permission_policy_remove_from_user_group_response import (
    PermissionPolicyRemoveFromUserGroupResponse as PermissionPolicyRemoveFromUserGroupResponse,
)
from .permission_policy_remove_from_service_user_response import (
    PermissionPolicyRemoveFromServiceUserResponse as PermissionPolicyRemoveFromServiceUserResponse,
)
from .permission_policy_remove_from_support_user_response import (
    PermissionPolicyRemoveFromSupportUserResponse as PermissionPolicyRemoveFromSupportUserResponse,
)
